@@ -16,7 +16,7 @@
 from collections import OrderedDict
 import functools
 import re
-from typing import Dict, Optional, Sequence, Tuple, Type, Union
+from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
 from google.api_core.client_options import ClientOptions
@@ -240,21 +240,20 @@ class Controller2AsyncClient:
         format. Agents must handle ``debuggee_id`` value changing upon
         re-registration.
 
-
         .. code-block:: python
 
             from google.cloud import debugger_v2
 
-            def sample_register_debuggee():
+            async def sample_register_debuggee():
                 # Create a client
-                client = debugger_v2.Controller2Client()
+                client = debugger_v2.Controller2AsyncClient()
 
                 # Initialize request argument(s)
                 request = debugger_v2.RegisterDebuggeeRequest(
                 )
 
                 # Make the request
-                response = client.register_debuggee(request=request)
+                response = await client.register_debuggee(request=request)
 
                 # Handle the response
                 print(response)
@@ -306,7 +305,12 @@ class Controller2AsyncClient:
         )
 
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
@@ -335,14 +339,13 @@ class Controller2AsyncClient:
         breakpoints that are completed until the controller removes them
         from the active list to avoid setting those breakpoints again.
 
-
         .. code-block:: python
 
             from google.cloud import debugger_v2
 
-            def sample_list_active_breakpoints():
+            async def sample_list_active_breakpoints():
                 # Create a client
-                client = debugger_v2.Controller2Client()
+                client = debugger_v2.Controller2AsyncClient()
 
                 # Initialize request argument(s)
                 request = debugger_v2.ListActiveBreakpointsRequest(
@@ -350,7 +353,7 @@ class Controller2AsyncClient:
                 )
 
                 # Make the request
-                response = client.list_active_breakpoints(request=request)
+                response = await client.list_active_breakpoints(request=request)
 
                 # Handle the response
                 print(response)
@@ -410,8 +413,21 @@ class Controller2AsyncClient:
             client_info=DEFAULT_CLIENT_INFO,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("debuggee_id", request.debuggee_id),)
+            ),
+        )
+
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
@@ -436,14 +452,13 @@ class Controller2AsyncClient:
         changes such as canonicalizing a value or snapping the location
         to the correct line of code.
 
-
         .. code-block:: python
 
             from google.cloud import debugger_v2
 
-            def sample_update_active_breakpoint():
+            async def sample_update_active_breakpoint():
                 # Create a client
-                client = debugger_v2.Controller2Client()
+                client = debugger_v2.Controller2AsyncClient()
 
                 # Initialize request argument(s)
                 request = debugger_v2.UpdateActiveBreakpointRequest(
@@ -451,7 +466,7 @@ class Controller2AsyncClient:
                 )
 
                 # Make the request
-                response = client.update_active_breakpoint(request=request)
+                response = await client.update_active_breakpoint(request=request)
 
                 # Handle the response
                 print(response)
@@ -525,8 +540,24 @@ class Controller2AsyncClient:
             client_info=DEFAULT_CLIENT_INFO,
         )
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (
+                    ("debuggee_id", request.debuggee_id),
+                    ("breakpoint.id", request.breakpoint_.id),
+                )
+            ),
+        )
+
         # Send the request.
-        response = await rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = await rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
