@@ -16,7 +16,7 @@
 from collections import OrderedDict
 import os
 import re
-from typing import Dict, Optional, Sequence, Tuple, Type, Union
+from typing import Dict, Mapping, Optional, Sequence, Tuple, Type, Union
 import pkg_resources
 
 from google.api_core import client_options as client_options_lib
@@ -53,7 +53,10 @@ class Controller2ClientMeta(type):
     _transport_registry["grpc"] = Controller2GrpcTransport
     _transport_registry["grpc_asyncio"] = Controller2GrpcAsyncIOTransport
 
-    def get_transport_class(cls, label: str = None,) -> Type[Controller2Transport]:
+    def get_transport_class(
+        cls,
+        label: str = None,
+    ) -> Type[Controller2Transport]:
         """Returns an appropriate transport class.
 
         Args:
@@ -180,7 +183,9 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         return self._transport
 
     @staticmethod
-    def common_billing_account_path(billing_account: str,) -> str:
+    def common_billing_account_path(
+        billing_account: str,
+    ) -> str:
         """Returns a fully-qualified billing_account string."""
         return "billingAccounts/{billing_account}".format(
             billing_account=billing_account,
@@ -193,9 +198,13 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_folder_path(folder: str,) -> str:
+    def common_folder_path(
+        folder: str,
+    ) -> str:
         """Returns a fully-qualified folder string."""
-        return "folders/{folder}".format(folder=folder,)
+        return "folders/{folder}".format(
+            folder=folder,
+        )
 
     @staticmethod
     def parse_common_folder_path(path: str) -> Dict[str, str]:
@@ -204,9 +213,13 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_organization_path(organization: str,) -> str:
+    def common_organization_path(
+        organization: str,
+    ) -> str:
         """Returns a fully-qualified organization string."""
-        return "organizations/{organization}".format(organization=organization,)
+        return "organizations/{organization}".format(
+            organization=organization,
+        )
 
     @staticmethod
     def parse_common_organization_path(path: str) -> Dict[str, str]:
@@ -215,9 +228,13 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_project_path(project: str,) -> str:
+    def common_project_path(
+        project: str,
+    ) -> str:
         """Returns a fully-qualified project string."""
-        return "projects/{project}".format(project=project,)
+        return "projects/{project}".format(
+            project=project,
+        )
 
     @staticmethod
     def parse_common_project_path(path: str) -> Dict[str, str]:
@@ -226,10 +243,14 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         return m.groupdict() if m else {}
 
     @staticmethod
-    def common_location_path(project: str, location: str,) -> str:
+    def common_location_path(
+        project: str,
+        location: str,
+    ) -> str:
         """Returns a fully-qualified location string."""
         return "projects/{project}/locations/{location}".format(
-            project=project, location=location,
+            project=project,
+            location=location,
         )
 
     @staticmethod
@@ -401,6 +422,7 @@ class Controller2Client(metaclass=Controller2ClientMeta):
                 quota_project_id=client_options.quota_project_id,
                 client_info=client_info,
                 always_use_jwt_access=True,
+                api_audience=client_options.api_audience,
             )
 
     def register_debuggee(
@@ -425,7 +447,6 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         format. Agents must handle ``debuggee_id`` value changing upon
         re-registration.
 
-
         .. code-block:: python
 
             from google.cloud import debugger_v2
@@ -491,7 +512,12 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         rpc = self._transport._wrapped_methods[self._transport.register_debuggee]
 
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
@@ -520,7 +546,6 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         breakpoints that are completed until the controller removes them
         from the active list to avoid setting those breakpoints again.
 
-
         .. code-block:: python
 
             from google.cloud import debugger_v2
@@ -585,8 +610,21 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         # and friendly error handling.
         rpc = self._transport._wrapped_methods[self._transport.list_active_breakpoints]
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (("debuggee_id", request.debuggee_id),)
+            ),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
@@ -611,7 +649,6 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         changes such as canonicalizing a value or snapping the location
         to the correct line of code.
 
-
         .. code-block:: python
 
             from google.cloud import debugger_v2
@@ -690,8 +727,24 @@ class Controller2Client(metaclass=Controller2ClientMeta):
         # and friendly error handling.
         rpc = self._transport._wrapped_methods[self._transport.update_active_breakpoint]
 
+        # Certain fields should be provided within the metadata header;
+        # add these here.
+        metadata = tuple(metadata) + (
+            gapic_v1.routing_header.to_grpc_metadata(
+                (
+                    ("debuggee_id", request.debuggee_id),
+                    ("breakpoint.id", request.breakpoint_.id),
+                )
+            ),
+        )
+
         # Send the request.
-        response = rpc(request, retry=retry, timeout=timeout, metadata=metadata,)
+        response = rpc(
+            request,
+            retry=retry,
+            timeout=timeout,
+            metadata=metadata,
+        )
 
         # Done; return the response.
         return response
